/*--------------------------------------------------------------------------
Copyright (c) 2010, Code Aurora Forum. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Code Aurora nor
      the names of its contributors may be used to endorse or promote
      products derived from this software without specific prior written
      permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--------------------------------------------------------------------------*/

#ifndef _VENC_TEST_PMEM_H
#define _VENC_TEST_PMEM_H

/*========================================================================

                     INCLUDE FILES FOR MODULE

==========================================================================*/
#include "OMX_Core.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <fcntl.h>
#include <linux/msm_q6venc.h>
#include <linux/android_pmem.h>
#include "venc_debug.h"

#include <sys/mman.h>


namespace venctest
{

  /**
   * @brief Class for allocating / deallocating pmem buffers.
   *
   * Underlying implementation is hidden. Optionally allocate a single pool of
   * memory and slice into multiple buffers.
   */
  class Pmem
  {
    public:

      /**
       * @brief Constructor
       */
      Pmem(OMX_S32 nBuffers);

      /**
       * @brief Destructor
       */
      ~Pmem();

    public:

      OMX_ERRORTYPE Allocate(OMX_U8** ppBuffer,
          OMX_S32 nBytes, int pmem_region_id);

      OMX_ERRORTYPE Free(OMX_U8* pBuffer);

    private:

      Pmem() {} // don't allow default constructor
      OMX_ERRORTYPE pmem_alloc(struct venc_pmem *pBuf, int size, int pmem_region_id);
      OMX_ERRORTYPE pmem_free(struct venc_pmem *pBuf);

    private:

      OMX_S32 m_nBuffers;
      OMX_S32 m_nBuffersAlloc;
      struct venc_pmem* m_pBufferInfo;

  };
} // namespace venctest

#endif // #ifndef _VENC_TEST_PMEM_H
